import React, { useState, useEffect } from 'react';
import Grid from '@mui/material/Grid';
import ShipmentPanel from '../components/ShipmentPanel';
import ShipmentDetails from '../components/ShipmentDetails';
import ShipmentForm from '../components/ShipmentForm';
import { Dewar, Shipment_Input, DefaultService, OpenAPI, ContactPerson } from '../../openapi';

type ShipmentViewProps = React.PropsWithChildren<Record<string, never>>;

const API_BASE_URL = 'http://127.0.0.1:8000';
OpenAPI.BASE = API_BASE_URL;

const ShipmentView: React.FC<ShipmentViewProps> = () => {
    const [isCreatingShipment, setIsCreatingShipment] = useState(false);
    const [selectedShipment, setSelectedShipment] = useState<Shipment_Input | null>(null);
    const [selectedDewar, setSelectedDewar] = useState<Dewar | null>(null);
    const [shipments, setShipments] = useState<Shipment_Input[]>([]);
    const [error, setError] = useState<string | null>(null);
    const [defaultContactPerson, setDefaultContactPerson] = useState<ContactPerson | undefined>();

    const fetchAndSetShipments = async () => {
        try {
            const shipmentsData: Shipment_Input[] = await DefaultService.getShipmentsShipmentsGet();
            shipmentsData.sort((a, b) => new Date(b.shipment_date).getTime() - new Date(a.shipment_date).getTime());
            setShipments(shipmentsData);
            console.log('Fetched and set shipments:', shipmentsData);
        } catch (error) {
            console.error('Failed to fetch shipments:', error);
            setError('Failed to fetch shipments. Please try again later.');
        }
    };

    const fetchDefaultContactPerson = async () => {
        try {
            const c: ContactPerson[] = await DefaultService.getContactsContactsGet();
            setDefaultContactPerson(c[0]);
        } catch {
            setError('Failed to load contact persons. Please try again later.');
        }
    };

    useEffect(() => {
        fetchAndSetShipments();
        fetchDefaultContactPerson();
    }, []);

    useEffect(() => {
        console.log('Updated shipments:', shipments);
    }, [shipments]);

    const handleSelectShipment = (shipment: Shipment_Input | null) => {
        setSelectedShipment(shipment);
        setIsCreatingShipment(false);
    };

    const handleCancelShipmentForm = () => {
        setIsCreatingShipment(false);
    };

    const renderShipmentContent = () => {
        if (isCreatingShipment) {
            return (
                <ShipmentForm
                    sx={{ flexGrow: 1 }}
                    onCancel={handleCancelShipmentForm}
                    refreshShipments={fetchAndSetShipments}
                />
            );
        }
        if (selectedShipment) {
            return (
                <ShipmentDetails
                    isCreatingShipment={isCreatingShipment}
                    sx={{ flexGrow: 1 }}
                    selectedShipment={selectedShipment}
                    selectedDewar={selectedDewar}
                    setSelectedDewar={setSelectedDewar}
                    setSelectedShipment={setSelectedShipment}
                    defaultContactPerson={defaultContactPerson}
                    refreshShipments={fetchAndSetShipments} // Ensure refreshShipments is passed here
                />
            );
        }
        return <div>No shipment details available.</div>;
    };

    return (
        <Grid container spacing={2} sx={{ height: '100vh' }}>
            <Grid
                item
                xs={12}
                md={3}
                sx={{
                    display: 'flex',
                    flexDirection: 'column',
                    flexGrow: 0,
                }}
            >
                <ShipmentPanel
                    selectedPage="Shipments"
                    setCreatingShipment={setIsCreatingShipment}
                    selectShipment={handleSelectShipment}
                    shipments={shipments}
                    refreshShipments={fetchAndSetShipments}
                    error={error}
                />
            </Grid>
            <Grid
                item
                xs={12}
                md={8}
                sx={{
                    display: 'flex',
                    flexDirection: 'column',
                    flexGrow: 1,
                }}
            >
                {renderShipmentContent()}
            </Grid>
        </Grid>
    );
};

export default ShipmentView;